local M = {}

local drivetrain = require("drivetrain")
local engine = require("engine")
local electrics = require("electrics")
local turbocharger = require("turbocharger")
local checkSounds = 0

M.brakeWear = 0
M.brakesDamaged = false
M.canStartEngine = true

M.ableToShift = true

M.engineIsOff = false
M.transmissionWear = 0
M.transmissionDamaged = false
M.targetShiftPosition = 0

local newShiftingEnabled = true --Testing at the moment, if you found this you can enable it to see what I've got so far (if it's unfinished)
local transmissionDamagedMsg = false

local blinkTimerThreshold = 0.4
local blinkTimer = 0

local whineSound = nil
local squealSound = nil
local parkEngageSound = nil
local startEngineSound = nil
local engineStartedSound = nil
local gearGrindSound = nil

local initializeTimer = 0

local justInitialized = true
local pitchSpeed = 0
local preParkingValue = nil

local startPressed = false
local startTimer = 0
local startTimer2 = 0
local startingVehicle = false
local vehicleName = ""
local stallRPM = 5

local function createSFXSource(filename, description, SFXProfileName, nodeID)
  local snd = gameEngine:createSFXSource(filename, description, SFXProfileName, true)
  obj:attachSFXNode(snd, nodeID)
  return snd
end


local function createSfx()
    whineSound = createSFXSource('art/sound/gear_whine1.ogg', 'AudioDefault3D', "GearWhineSound1", 1)
    squealSound = createSFXSource('art/sound/brake_squeal.ogg', 'AudioDefault3D', "BrakeSquealSound", 1)
    parkEngageSound = createSFXSource('art/sound/parking_brake_engage.ogg', 'AudioDefault3D', "ParkingBrakeEngage", 1)
    startEngineSound = createSFXSource('art/sound/start_engine.ogg', 'AudioDefault3D', "StartEngineSound", 1)
    engineStartedSound = createSFXSource('art/sound/engine_started.ogg', 'AudioDefault3D', "EngineStartedSound", 1)
    gearGrindSound = createSFXSource('art/sound/gear_grind.ogg', 'AudioDefault3D', "GearGrindSound", 1)
end

local function playPitchSound(sfxsrc, soundName, nodeID, volume, pitch) 
	obj:setPitch(sfxsrc.clip, pitch)
	obj:playSFXOnce(soundName, nodeID, volume, pitch)
end


local function generateBlinkPulse(dt)
    blinkTimer = blinkTimer + dt
    if blinkTimer > blinkTimerThreshold then
    	if(electrics.values.signal_right_input == 1 or electrics.values.signal_left_input == 1) then
	        sounds.playSoundOnceAtNode("ShiftTestSound", 1, 1)
    	end
        blinkTimer = 0
    end
end

local usingGamepad = false --Button Clutch instead of axis

local function stallTimes()
	if(vehicleName == "Gavril D-Series") then
		stallRPM = 500
	elseif(vehicleName == "Gavril H-Series") then
		stallRPM = 500
	elseif(vehicleName == "Gavril T-Series") then
		stallRPM = 150
	elseif(vehicleName == "Gavril Grand Marshal") then
		stallRPM = 500
	elseif(vehicleName == "Gavril Roamer") then
		stallRPM = 500
	elseif(vehicleName == "Gavril Barstow") then
		stallRPM = 450
	elseif(vehicleName == "Burnside Special") then
		stallRPM = 380
	elseif(vehicleName == "Ibishu Covet") then
		stallRPM = 450
	elseif(vehicleName == "Ibishu 200BX") then
		stallRPM = 500
	elseif(vehicleName == "Ibishu Pigeon") then
		stallRPM = 250
	elseif(vehicleName == "Ibishu Pessima") then
		stallRPM = 500
	elseif(vehicleName == "Ibishu Miramar") then
		stallRPM = 260
	elseif(vehicleName == "Hirochi Sunburst") then
		stallRPM = 500
	elseif(vehicleName == "Hirochi SBR4") then
		stallRPM = 260
	elseif(vehicleName == "Bruckell Moonhawk") then
		stallRPM = 420
	elseif(vehicleName == "Civetta Bolide") then
		stallRPM = 420
	elseif(vehicleName == "ETK 800 Series") then
		stallRPM = 420
	else
		stallRPM = 250
	end

	if(usingGamepad) then
		stallRPM = stallRPM - 240
	end
end

local function init()
	createSfx()
	M.targetShiftPosition = 0
	M.engineIsOff = false
	justInitialized = true
	startPressed = false
	startingVehicle = false
	startTimer = 0
	startTimer2 = 0
	initializeTimer = 0
	if(electrics.values.parkingbrake) then preParkingValue = math.floor(electrics.values.parkingbrake) end
	vehicleName = v.main.information.name

	stallTimes()
end

local function shiftUp()
	local eng = v.data.engine
	if(M.ableToShift or M.targetShiftPosition + 1 == 0) then
    	M.targetShiftPosition = M.targetShiftPosition + 1
	else
		M.transmissionWear = M.transmissionWear + 5
		playPitchSound(gearGrindSound, "GearGrindSound", 1, 1, 1) --speed / 70
	end
	M.targetShiftPosition = math.min( math.max(M.targetShiftPosition, -eng.revGearCount), eng.fwdGearCount )
	gui.message("Target Shift: ".. M.targetShiftPosition)
end

local function shiftDown()
	local eng = v.data.engine
	if(M.ableToShift or M.targetShiftPosition - 1 == 0) then
    	M.targetShiftPosition = M.targetShiftPosition - 1
	else
		M.transmissionWear = M.transmissionWear + 5
		playPitchSound(gearGrindSound, "GearGrindSound", 1, 1, 1) --speed / 70
	end
	M.targetShiftPosition = math.min( math.max(M.targetShiftPosition, -eng.revGearCount), eng.fwdGearCount )
	gui.message("Target Shift: ".. M.targetShiftPosition)
end

local function shiftToGear(val)
	if(M.ableToShift or val == 0) then
   		M.targetShiftPosition = val
	else
		M.transmissionWear = M.transmissionWear + 5
		playPitchSound(gearGrindSound, "GearGrindSound", 1, 1, 1) --speed / 70
	end
	M.targetShiftPosition = math.min( math.max(M.targetShiftPosition, -eng.revGearCount), eng.fwdGearCount )
	gui.message("Target Shift: ".. M.targetShiftPosition)
end

local function update(dt)
	local eng = v.data.engine
	local engineRPM = drivetrain.engineAV  * 9.5493


	if(newShiftingEnabled and drivetrain.shifterMode == 0 or drivetrain.shifterMode == 1) then
		
		M.ableToShift = false

		--Limit the shift position
        M.targetShiftPosition = math.min( math.max(M.targetShiftPosition, -eng.revGearCount), eng.fwdGearCount )
		--[[

		    local throttleCubed = drivetrain.throttle * drivetrain.throttle * drivetrain.throttle
		    local shiftDown = (eng.lowShiftDownAV + (eng.highShiftDownAV - eng.lowShiftDownAV) * throttleCubed) * 10
		    local shiftUp   = (eng.lowShiftUpAV + (eng.highShiftUpAV - eng.lowShiftUpAV) * throttleCubed) * 10
		]]--
		drivetrain.shiftUp = shiftUp
		drivetrain.shiftDown = shiftDown
		drivetrain.shiftToGear = shiftToGear
		drivetrain.shifterPosition = M.targetShiftPosition
		
	    if(input.clutch > 0 or M.engineIsOff and M.transmissionWear < 100) then
	    	M.ableToShift = true
	    elseif(M.transmissionWear >= 100 and not transmissionDamagedMsg) then
	    	gui.message("Transmission Damaged")
	    	transmissionDamagedMsg = true
	    end
	end

	if(M.engineIsOff) then
		drivetrain.engineAV = 0 --Cheatsy way of doing an engine off ;D
	end

	M.canStartEngine = true
    for _, n in ipairs(v.data.engine.waterDamage.nodes) do
        if obj:inWater(n) then
        	M.canStartEngine = false
        end
    end
	M.brakeWear = M.brakeWear + ((drivetrain.avgAV / 120000) * drivetrain.brake)
	generateBlinkPulse(dt)
	local speed = obj:getVelocity():length() * 2.236 -- speed
	checkSounds = checkSounds + dt
	if(checkSounds > 0.2) then

		--Parking Brake Engage
		if(preParkingValue ~= math.floor(electrics.values.parkingbrake) and math.floor(electrics.values.parkingbrake) == 1) then 
	        sounds.playSoundOnceAtNode("ParkingBrakeEngage", 1, 1)
			preParkingValue = math.floor(electrics.values.parkingbrake) 
		elseif( math.floor(electrics.values.parkingbrake) ~= 1) then
			preParkingValue = math.floor(electrics.values.parkingbrake) 
		end

		--Whining
	    if(drivetrain.gear == -1) then 
	    	if(speed > 40) then pitchSpeed = 40 else pitchSpeed = speed end
	        playPitchSound(whineSound, "GearWhineSound1", 1, speed / 70, 1 + pitchSpeed / 650) --speed / 70
	    end

	    --Brake Squeal
	    if(drivetrain.brake > 0 and M.brakeWear > 25 and (drivetrain.avgAV > 4 or drivetrain.avgAV < -4)) then
	        playPitchSound(squealSound, "BrakeSquealSound", 1, drivetrain.brake / 1.5, 1)
	    end

	    checkSounds = 0
	end

	if(not startingVehicle and engineRPM < stallRPM and initializeTimer > 1890 and drivetrain.shifterMode ~= 2 and drivetrain.shifterMode ~= 3) then
		M.engineIsOff = true
	end

	if(startingVehicle and M.engineIsOff) then
		startTimer2 = startTimer2 + dt
		startTimer = startTimer + 1
		if(startTimer2 > 0.15) then
			if(startTimer < 3000 or not M.canStartEngine) then
				playPitchSound(startEngineSound, "StartEngineSound", 1, 1, 1 + math.sin(startTimer / 5) / 10) --speed / 70
			elseif M.canStartEngine then 
				drivetrain.engineAV = 150
				M.engineIsOff = false
				playPitchSound(engineStartedSound, "EngineStartedSound", 1, 1, 1) --speed / 70
				startTimer = 0
				startTimer2 = 0
				startingVehicle = false
			end
			startTimer2 = 0
		end
	end
	initializeTimer = initializeTimer + 1

	if(engine.oilOverheatDamage > (v.data.engine.connectingRodDamageThreshold or 2000000)) then
		M.canStartEngine = false
	end
	
	
	if(startingVehicle and not M.engineIsOff) then
		M.engineIsOff = true
	end

	if(input.keys.E and not startPressed and not startingVehicle) then
		startingVehicle = true
		startPressed = true
	end

	if((drivetrain.avgAV > 3 or drivetrain.avgAV < -3) and M.engineIsOff and drivetrain.gear ~= 0) then
		M.engineIsOff = false
	end

	if(not input.keys.E) then
		startingVehicle = false
		startTimer = 0
		startTimer2 = 0
		startPressed = false
	end

	if(justInitialized) then
		M.engineIsOff = false
		justInitialized = false
	end
end


M.init = init
M.update = update

return M